#include <assert.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#include "pass2.h"

extern void defalign(int);

char *exname(char *p);

char *rnames[] = {
	"r0", "r1", "r2", "r3","r4","r5", "r6", "r7",
	"r8", "r9", "r10", "r11","r12","r13", "r14", "r15",
	"r16", "r17", "r18", "r19","r20","r21", "r22", "r23",
	"r24", "r25", "r26", "r27","r28","r29", "fp", "sp",
};

/*
 * Print out a label of number label.
 */
/* mip/reader.c:311 */
void deflab(int label)
{
  printf(LABFMT ":\n", label);
}

/*
 * Print out the prolog for a function.
 */
/* mip/reader.c:303 */
void prologue(struct interpass_prolog *ipp)
{
  printf(".%s: ; function\n", ipp->ipp_name);
}

/*
 * End-of-function; the opposite of prolog().
 */
/* mip/reader.c:306 */
void eoftn(struct interpass_prolog *ipp)
{
}
/*
 * Called for an O macro in the table, o is the opcode and
 * f is the argument.  Supposed to write out simple ops.
 */
/* mip/match.c:320 */
void hopcode(int f, int o)
{
  //printf("hopcode f:%d, o:%d\n", f, o );
}
	
/*
 * Return type size in bytes.  Used by R2REGS, arg 2 to offset()
 */
/* cc/ccom/symtabs.c:88 */
int tlen(NODE *p)
{
  switch(p->n_type) {
  case CHAR:
  case UCHAR:
    return (1);
  case SHORT:
  case USHORT:
    return (SZSHORT/SZCHAR);
  case DOUBLE:
    return (SZDOUBLE/SZCHAR);
  case INT:
  case UNSIGNED:
  case LONG:
  case ULONG:
    return (SZINT/SZCHAR);
  case LONGLONG:
  case ULONGLONG:
    return (SZLONGLONG/SZCHAR);
  default:
    if(!ISPTR(p->n_type))
      comperr("tlen type %d not pointer");
    return SZPOINT(p->n_type)/SZCHAR;
  }
}

/*
 * Field ops are found in the table (S, H, M or N), and
 * this function may alter the default behaviour of printing
 * out the values.  Return 1 if handled.
 */
/* mip/match.c:290 */
int fldexpand(NODE *p, int cookie, char **cp)
{
  return 1;
}
/*
 * Z directive in table, c is the argument.  Lets the 
 * target do anything.
 */
/* mip/match.c:277 */
void zzzcode(NODE *p, int c)
{
  /*NODE *l;*/
  switch(c) {
  case 'C': /* Print label address */
    p = p->n_left;
    if (p->n_lval)
      printf(LABFMT, (int)p->n_lval);
    else
      printf("%s", p->n_name);
    break;

  default:
    printf("bad zzzcode %c\n", c);    
  }
}

/*
 * Check if a node should be rewritten in ffld().  Only
 * called if FIELDOPS not defined.
*/
/* mip/reader.c:855 */
int rewfld(NODE *p)
{
  return 1;
}

/*
 * Check if the bitfield shape matches this node when
 * found in the table.
 */
/* mip/match.c159 */
int flshape(NODE *p)
{
  return SRREG; /* put it into a register */
}

/*
 * Check INTEMP shapes.  XXX should go now?
 */
/* mip/match.c:137 */
int shtemp(NODE *p)
{
  int op = p->n_op;
  if(op == OREG || op == REG || op == NAME)
    return SRDIR; /* direct match */
  if(op == UMUL && shumul(p->n_left))
    return SROREG; /* convert into oreg */
  return SRREG; /* put it into a register */
}
/*
 * Print out constants for M or N bitfield modifier.
 */
/* mip/match.c:309 */
void adrcon(CONSZ val)
{
  printf(CONFMT, val);
}

/*
 * Print out constant for C table directive.
 */
/* mip/match/c:330 */
void conput(FILE *fp, NODE *p)
{
  int val = p->n_lval;
  
  switch (p->n_op) {
  case ICON:
    if (p->n_name[0] != '\0') {
      fprintf(fp, "%s", p->n_name);
      if (p->n_lval)
	fprintf(fp, "+%d", val);
    } else
      fprintf(fp, "%d", val);
    return;    
  default:
    comperr("illegal conput");
  }
}

/*
 * Called for an I table directive.
 */
/* mip/match.c:334 */
void insput(NODE *p) 
{}
	
/*
 * Print out the upper part of an address; U table directive.
 */
/* mip/match.c:342 */
void upput(NODE *p, int size)	
{
}

/*
 * Print out the lower (or only) part of an address; 
 * A table directive.
 */		
/* mip/match.c:338 */
void adrput(FILE *io, NODE *p)
{
  if (p->n_op == FLD)
    p = p->n_left;
  switch (p->n_op) {
  case NAME:
    if (p->n_name[0] != '\0') {
      fputs(p->n_name, io);
      if (p->n_lval != 0)
	fprintf(io, "+%lld", p->n_lval);
    } else
      fprintf(io, CONFMT, p->n_lval);
    return;
  case OREG:
    if(p->n_lval)
      fprintf(io, "%d", (int)p->n_lval);
    fprintf(io, " (%s)", rnames[p->n_rval]);
    return;    
  case ICON:
    /* addressable value of the constant */
    fputc('#', io);
    conput(io, p);
    return;
  case REG:
    fprintf(io, "%s", rnames[p->n_rval]);
    return;
  default:
    comperr("illegal address, op %d, node %p", p->n_op, p);
    return;
  }
}
/*
 * Print out conditional branch o.  Called when a CBRANCH
 * node is found after an op that results in FORCC has
 * been emitted.
 */
void cbgen(int o, int lab)
{
}

/*
 * Directly when the whole list of trees for a function is
 * read into pass2 the list is given myreader() to let the
 * target do something with it.
 */
/* mip/reader.c:218 */
void myreader(struct interpass *ipole)
{
}

/*
 * Called from canon() to do target-specific stuff with
 * the tree p.
 */
/* mip/reader.c:1069 */
void mycanon(NODE *p)
{
}

/*
 * Called from optimize() with the linked list to do
 * target-dependent optimizations.
*/
/* mip/optim2.c:242 */
void myoptim(struct interpass *ip)
{
}

/*
 * Print out a move instruction from register s to d, 
 * carrying data of type t.
 */
/* mip/reader.c:539 */
void rmove(int s, int d, TWORD t)
{
}

/* 
 * For register class c, find worst-case displacement of
 * the number of registers in the array r[] indexed by class.
 */
/* mip/regs.c:390 */
int COLORMAP(int c, int *r)
{
  int num = 0;
  switch(c){
  case CLASSA:
    num   += r[CLASSA];
    return num < 30;
  case CLASSB:
    num   += r[CLASSB];
    return num < 2;    
  default:
    break;
  }
  return 0; /* XXX gcc */
}
/*
 * Return a class suitable for a specific type.
 */
/* mip/regs.c:182 */
int gclass(TWORD t)
{
  return CLASSA;
}

/*
 * Called just before a function call is emitted.
 */
/* mip/reader.c:662 */
void lastcall(NODE *p)
{
}

/*
 * Try to match node p against shape shape.
 */
int special(NODE *p, int shape)
{
  return SRNOPE;
}	

/*
 * Target-dependent command-line options
 */
void mflags(char *str)
{
}
